/*
    File:               argonAboutBox.cpp
    Description:        Provides a simple AboutBox.
                        This file is part of the ArgonLibrary package.

    Author:             Roman Komary
    created on:         16.04.2006
    last updated:       16.09.2006 (Roman Komary)

    Copyright (c) 2006, Roman Komary. All rights reserved.

    Distributed under the Boost Software License, Version 1.0. (See
    accompanying file LICENSE_1_0.ArgonLibrary.txt or copy at
    http://www.boost.org/LICENSE_1_0.txt)
*/

#define __argonAboutBox_cpp
#define __an_ArgonLibrary_cpp       // helpful flag, if you need to make conditional compilations e.g. in "stdafx.h"

#include "stdafx.h"
#include "argonAboutBox.h"

namespace ArgonLibrary
{




// Language Table strings for the AboutBox

const LangTableDesc g_AboutBoxLangDescs[] = {
    #ifndef _argon_no_LangEnglish
    // English
    { abbid_About,                  LANG_ENGLISH,   ltmapt_Direct,  (LONG_PTR) _T("About") },
    { abbid_Version,                LANG_ENGLISH,   ltmapt_Direct,  (LONG_PTR) _T("version") },
    { abbid_Build,                  LANG_ENGLISH,   ltmapt_Direct,  (LONG_PTR) _T("Build") },
    #endif

    #ifndef _argon_no_LangGerman
    // German
    { abbid_About,                  LANG_GERMAN,    ltmapt_Direct,  (LONG_PTR) _T("ber") },
    { abbid_Version,                LANG_GERMAN,    ltmapt_Direct,  (LONG_PTR) _T("Version") },
    { abbid_Build,                  LANG_GERMAN,    ltmapt_Direct,  (LONG_PTR) _T("Build") },
    #endif

    #ifndef _argon_no_LangItalian
    // Italian
    { abbid_About,                  LANG_ITALIAN,   ltmapt_Direct,  (LONG_PTR) _T("Per") },
    { abbid_Version,                LANG_ITALIAN,   ltmapt_Direct,  (LONG_PTR) _T("version") },
    { abbid_Build,                  LANG_ITALIAN,   ltmapt_Direct,  (LONG_PTR) _T("Build") },
    #endif

    LANGTABLEDESC_ENDMARK
};

AboutBoxLanguageTable                            g_AboutBoxLanguageTable;
LanguageDescriptionTableT<AboutBoxLanguageTable> g_AboutBoxLanguageDescriptionTable( g_AboutBoxLangDescs );




///<param name="dwMmbFlags"> combination of MMB_* flags </param>
void AboutBox::DoModal( HWND hwndParent,
                        HMODULE hModule,
                        PCTSTR pstrIconID_or_Resname,
                        PCTSTR pstrAdditionalText,
                        DWORD dwMmbFlags )
{
    String msg = getVersionInfo();

    msg += _T("\n\n");
    msg += pstrAdditionalText;
    msg += _T('\n');

    Version v;
    String cap = AboutBoxLanguageTable::getStringThreadLocale( abbid_About );
    cap += _T(' ');
    cap += v.Get(Tverstr_ProductName);

    m_Mmb.MessageBox( hwndParent, msg, NULL, cap,
                      MB_OK, dwMmbFlags, GetThreadLocale(),
                      NULL, NULL, NULL, NULL,
                      hModule ? hModule : Get_Resource_Instance(),
                      (UINT_PTR) pstrIconID_or_Resname );
}

String AboutBox::getVersionInfo()
{
    Version v;
    String s = v.Get(Tverstr_ProductName);
    s += _T(' ');
    s += AboutBoxLanguageTable::getStringThreadLocale( abbid_Version );
    s += _T(' ');
    s += String(v.Get(Tverstr_ProductVersion)).BeautifyVersionNr();
    s += _T("\n\n");

    s += AboutBoxLanguageTable::getStringThreadLocale( abbid_Build );
    s += _T(": ");
    s += String(v.Get(Tverstr_ProductVersion)).GetBuildNr();
    s += _T(" (");
    s += _T(__DATE__);
    s += _T(")\n\n");

    s += v.Get(Tverstr_LegalCopyright);

    return s;
}




}
