/*
    File:               argonAboutBox.h
    Description:        Provides a simple AboutBox.
                        This file is part of the ArgonLibrary package.

    Author:             Roman Komary
    created on:         16.04.2006
    last updated:       18.09.2006 (Roman Komary)

    Copyright (c) 2006, Roman Komary. All rights reserved.

    Distributed under the Boost Software License, Version 1.0. (See
    accompanying file LICENSE_1_0.ArgonLibrary.txt or copy at
    http://www.boost.org/LICENSE_1_0.txt)
*/

/*
    Switches:

    _argon_no_Lang...       These defines allow excluding of specific languages predefined
                            in the ArgonLibrary.
                            By default, all languages available in the ArgonLibrary will
                            be included. If you define one or some of this set of defines,
                            the appropriate language strings will not be included in
                            the compilation.
                            The following languages and their excluding defines are available:
                                _argon_no_LangEnglish
                                _argon_no_LangGerman
                                _argon_no_LangItalian
*/

#ifndef __argonAboutBox_h
#define __argonAboutBox_h

#ifndef __ArgonLibrary__
#define __ArgonLibrary__            // helpful flag, if you need to make conditional compilations somewhere in your sourcecode
#endif

#ifndef _argon_no_includes
    #include "argonString.h"
    #include "argonVersion.h"
    #include "argonMoreMessageBox.h"
#endif

namespace ArgonLibrary
{




// Language Table strings for the AboutBox

class AboutBoxLanguageTable_Def {  };
typedef LanguageTableT<AboutBoxLanguageTable_Def> AboutBoxLanguageTable;

// language table string IDs for the MoreMessageBox (Buttons etc.).
enum AboutBoxIDs
{
    abbid_About = 1,
    abbid_Version,
    abbid_Build
};

class AboutBox
{
    public :
        MoreMessageBox m_Mmb;

    public :
        AboutBox() { m_Mmb.m_minWidthAsNumButtons = 3; }

        void DoModal( UINT idIcon = 0,
                      PCTSTR pstrAdditionalText = NULL )
            { DoModal( NULL, NULL, MAKEINTRESOURCE(idIcon), pstrAdditionalText ); }
        void DoModal( PCTSTR pstrIconID_or_Resname,
                      PCTSTR pstrAdditionalText = NULL )
            { DoModal( NULL, NULL, pstrIconID_or_Resname, pstrAdditionalText ); }
        void DoModal( HWND hwndParent,
                      HMODULE hModule,
                      PCTSTR pstrIconID_or_Resname,
                      PCTSTR pstrAdditionalText = NULL,
                      DWORD dwMmbFlags = MMB_MsgIcon_ExtraOffsetY | MMB_CenterOverParent );

        static String getVersionInfo();
};




}

#endif
