/*
    File:               argonKernel32Dll.cpp
    Description:        Parts of the Kernel Dll API.
                        Dynamically loading of some functions of the Kernel Dll API.
                        This file is part of the ArgonLibrary package.

    Author:             Roman Komary
    created on:         01.12.2005
    last updated:       07.02.2006 (Roman Komary)

    Copyright (c) 2005-2006, Roman Komary.

    Distributed under the Boost Software License, Version 1.0. (See
    accompanying file LICENSE_1_0.ArgonLibrary.txt or copy at
    http://www.boost.org/LICENSE_1_0.txt)
*/

#define __argonKernel32Dll_cpp
#define __an_ArgonLibrary_cpp       // helpful flag, if you need to make conditional compilations e.g. in "stdafx.h"

#include "stdafx.h"
#include "argonKernel32Dll.h"

#ifndef _argon_no_includes
    #include "argonLocking.h"
#endif

namespace ArgonLibrary
{




BOOL WINAPI argonGetComputerNameExA(
    COMPUTER_NAME_FORMAT NameType,
    LPSTR lpBuffer,
    LPDWORD nSize
)
{
    Kernel32Dll::LoadAPI();
    if(   !Kernel32Dll::s_pfnGetComputerNameExA ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return Kernel32Dll::s_pfnGetComputerNameExA( NameType, lpBuffer, nSize );
}

BOOL WINAPI argonGetComputerNameExW(
    COMPUTER_NAME_FORMAT NameType,
    LPWSTR lpBuffer,
    LPDWORD nSize
)
{
    Kernel32Dll::LoadAPI();
    if(   !Kernel32Dll::s_pfnGetComputerNameExW ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return Kernel32Dll::s_pfnGetComputerNameExW( NameType, lpBuffer, nSize );
}

VOID WINAPI argonGetNativeSystemInfo(
    LPSYSTEM_INFO lpSystemInfo
)
{
    Kernel32Dll::LoadAPI();
    if(   !Kernel32Dll::s_pfnGetNativeSystemInfo ) return;
    return Kernel32Dll::s_pfnGetNativeSystemInfo( lpSystemInfo );
}

UINT WINAPI argonGetSystemWindowsDirectoryA(
    LPSTR lpBuffer,
    UINT uSize
)
{
    Kernel32Dll::LoadAPI();
    if(   !Kernel32Dll::s_pfnGetSystemWindowsDirectoryA ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return 0; }
    return Kernel32Dll::s_pfnGetSystemWindowsDirectoryA( lpBuffer, uSize );
}

UINT WINAPI argonGetSystemWindowsDirectoryW(
    LPWSTR lpBuffer,
    UINT uSize
)
{
    Kernel32Dll::LoadAPI();
    if(   !Kernel32Dll::s_pfnGetSystemWindowsDirectoryW ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return 0; }
    return Kernel32Dll::s_pfnGetSystemWindowsDirectoryW( lpBuffer, uSize );
}

UINT WINAPI argonGetSystemWow64DirectoryA(
    LPSTR lpBuffer,
    UINT uSize
)
{
    Kernel32Dll::LoadAPI();
    if(   !Kernel32Dll::s_pfnGetSystemWow64DirectoryA ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return 0; }
    return Kernel32Dll::s_pfnGetSystemWow64DirectoryA( lpBuffer, uSize );
}

UINT WINAPI argonGetSystemWow64DirectoryW(
    LPWSTR lpBuffer,
    UINT uSize
)
{
    Kernel32Dll::LoadAPI();
    if(   !Kernel32Dll::s_pfnGetSystemWow64DirectoryW ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return 0; }
    return Kernel32Dll::s_pfnGetSystemWow64DirectoryW( lpBuffer, uSize );
}

BOOL WINAPI argonIsProcessorFeaturePresent(
    DWORD ProcessorFeature
)
{
    Kernel32Dll::LoadAPI();
    if(   !Kernel32Dll::s_pfnIsProcessorFeaturePresent ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return Kernel32Dll::s_pfnIsProcessorFeaturePresent( ProcessorFeature );
}

BOOL WINAPI argonIsWow64Process(
    HANDLE hProcess,
    PBOOL Wow64Process
)
{
    Kernel32Dll::LoadAPI();
    if(   !Kernel32Dll::s_pfnIsWow64Process ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return Kernel32Dll::s_pfnIsWow64Process( hProcess, Wow64Process );
}

BOOL WINAPI argonSetComputerNameExA(
    COMPUTER_NAME_FORMAT NameType,
    LPCSTR lpBuffer
)
{
    Kernel32Dll::LoadAPI();
    if(   !Kernel32Dll::s_pfnSetComputerNameExA ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return Kernel32Dll::s_pfnSetComputerNameExA( NameType, lpBuffer );
}

BOOL WINAPI argonSetComputerNameExW(
    COMPUTER_NAME_FORMAT NameType,
    LPCWSTR lpBuffer
)
{
    Kernel32Dll::LoadAPI();
    if(   !Kernel32Dll::s_pfnSetComputerNameExW ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return Kernel32Dll::s_pfnSetComputerNameExW( NameType, lpBuffer );
}




Kernel32Dll::Unloader           Kernel32Dll::s_Unloader;
HMODULE                         Kernel32Dll::s_hModule                          = NULL;
TfnGetComputerNameExA           Kernel32Dll::s_pfnGetComputerNameExA            = NULL;
TfnGetComputerNameExW           Kernel32Dll::s_pfnGetComputerNameExW            = NULL;
TfnGetNativeSystemInfo          Kernel32Dll::s_pfnGetNativeSystemInfo           = NULL;
TfnGetSystemWindowsDirectoryA   Kernel32Dll::s_pfnGetSystemWindowsDirectoryA    = NULL;
TfnGetSystemWindowsDirectoryW   Kernel32Dll::s_pfnGetSystemWindowsDirectoryW    = NULL;
TfnGetSystemWow64DirectoryA     Kernel32Dll::s_pfnGetSystemWow64DirectoryA      = NULL;
TfnGetSystemWow64DirectoryW     Kernel32Dll::s_pfnGetSystemWow64DirectoryW      = NULL;
TfnIsProcessorFeaturePresent    Kernel32Dll::s_pfnIsProcessorFeaturePresent     = NULL;
TfnIsWow64Process               Kernel32Dll::s_pfnIsWow64Process                = NULL;
TfnSetComputerNameExA           Kernel32Dll::s_pfnSetComputerNameExA            = NULL;
TfnSetComputerNameExW           Kernel32Dll::s_pfnSetComputerNameExW            = NULL;
bool                            Kernel32Dll::s_bAutoUnload                      = true;

Kernel32Dll::Kernel32Dll()
{
}

// Remarks: We don't need any object of #Kernel32Dll#.
//  We just need its static members.



void Kernel32Dll::LoadAPI()
{
    if( s_hModule ) return;

    LockModuleWide lmw;

    if( s_hModule ) return;

    Nullify();

    s_hModule = ::LoadLibrary( _T(KERNEL32DLL_NAME) );

    if( s_hModule )
    {
        s_pfnGetComputerNameExA         = (TfnGetComputerNameExA        ) GetProcAddress( s_hModule, "GetComputerNameExA"         );
        s_pfnGetComputerNameExW         = (TfnGetComputerNameExW        ) GetProcAddress( s_hModule, "GetComputerNameExW"         );
        s_pfnGetNativeSystemInfo        = (TfnGetNativeSystemInfo       ) GetProcAddress( s_hModule, "GetNativeSystemInfo"        );
        s_pfnGetSystemWindowsDirectoryA = (TfnGetSystemWindowsDirectoryA) GetProcAddress( s_hModule, "GetSystemWindowsDirectoryA" );
        s_pfnGetSystemWindowsDirectoryW = (TfnGetSystemWindowsDirectoryW) GetProcAddress( s_hModule, "GetSystemWindowsDirectoryW" );
        s_pfnGetSystemWow64DirectoryA   = (TfnGetSystemWow64DirectoryA  ) GetProcAddress( s_hModule, "GetSystemWow64DirectoryA"   );
        s_pfnGetSystemWow64DirectoryW   = (TfnGetSystemWow64DirectoryW  ) GetProcAddress( s_hModule, "GetSystemWow64DirectoryW"   );
        s_pfnIsProcessorFeaturePresent  = (TfnIsProcessorFeaturePresent ) GetProcAddress( s_hModule, "IsProcessorFeaturePresent"  );
        s_pfnIsWow64Process             = (TfnIsWow64Process            ) GetProcAddress( s_hModule, "IsWow64Process"             );
        s_pfnSetComputerNameExA         = (TfnSetComputerNameExA        ) GetProcAddress( s_hModule, "SetComputerNameExA"         );
        s_pfnSetComputerNameExW         = (TfnSetComputerNameExW        ) GetProcAddress( s_hModule, "SetComputerNameExW"         );
    }
}

void Kernel32Dll::UnloadAPI()
{
    LockModuleWide lmw;

    if( s_hModule )
        ::FreeLibrary( s_hModule );

    Nullify();
}



void Kernel32Dll::Nullify()
{
    s_pfnGetComputerNameExA         = NULL;
    s_pfnGetComputerNameExW         = NULL;
    s_pfnGetNativeSystemInfo        = NULL;
    s_pfnGetSystemWindowsDirectoryA = NULL;
    s_pfnGetSystemWindowsDirectoryW = NULL;
    s_pfnGetSystemWow64DirectoryA   = NULL;
    s_pfnGetSystemWow64DirectoryW   = NULL;
    s_pfnIsProcessorFeaturePresent  = NULL;
    s_pfnIsWow64Process             = NULL;
    s_pfnSetComputerNameExA         = NULL;
    s_pfnSetComputerNameExW         = NULL;
    s_hModule = NULL;
}




}
