/*
    File:               argonVersion.h
    Description:        Application/Project Version Information.
                        This file is part of the ArgonLibrary package.

    Author:             Roman Komary
    created on:         5.10.2003
    last updated:       27.09.2006 (Roman Komary)

    Copyright (c) 2003-2006, Roman Komary.

    Distributed under the Boost Software License, Version 1.0. (See
    accompanying file LICENSE_1_0.ArgonLibrary.txt or copy at
    http://www.boost.org/LICENSE_1_0.txt)
*/

#ifndef __argonVersion_h
#define __argonVersion_h

#ifndef __ArgonLibrary__
#define __ArgonLibrary__            // helpful flag, if you need to make conditional compilations somewhere in your sourcecode
#endif

#ifndef _argon_no_includes
    #include "argonInstance.h"
    #include "argonString.h"
#endif

namespace ArgonLibrary
{




#ifndef _countof
#define _countof(array) (sizeof(array) / sizeof((array)[0]))
#endif

#define __W(x)          L ## x
#define _W(x)           __W(x)
// Note: To make #_W# macro work, we must do the detour using
//  the #__W# macro. This is necessary if you pass the #_W# macro
//  a macro-name instead of a string directly, so that #__W# is called
//  with the *expanded* macro-name, expanded to its defined string.

// For the Averstr_*, you can either directly use
// Averstr_* for ASCII usage, _W(Averstr_*) for Unicode usage,
// or use Tverstr_* for automatic selection.

#define Averstr_Comments            "Comments"
#define Averstr_CompanyName         "CompanyName"
#define Averstr_FileDescription     "FileDescription"
#define Averstr_FileVersion         "FileVersion"
#define Averstr_InternalName        "InternalName"
#define Averstr_LegalCopyright      "LegalCopyright"
#define Averstr_LegalTrademarks     "LegalTrademarks"
#define Averstr_OriginalFilename    "OriginalFilename"
#define Averstr_PrivateBuild        "PrivateBuild"
#define Averstr_ProductName         "ProductName"
#define Averstr_ProductVersion      "ProductVersion"
#define Averstr_SpecialBuild        "SpecialBuild"

#define Tverstr_Comments            _T(Averstr_Comments        )
#define Tverstr_CompanyName         _T(Averstr_CompanyName     )
#define Tverstr_FileDescription     _T(Averstr_FileDescription )
#define Tverstr_FileVersion         _T(Averstr_FileVersion     )
#define Tverstr_InternalName        _T(Averstr_InternalName    )
#define Tverstr_LegalCopyright      _T(Averstr_LegalCopyright  )
#define Tverstr_LegalTrademarks     _T(Averstr_LegalTrademarks )
#define Tverstr_OriginalFilename    _T(Averstr_OriginalFilename)
#define Tverstr_PrivateBuild        _T(Averstr_PrivateBuild    )
#define Tverstr_ProductName         _T(Averstr_ProductName     )
#define Tverstr_ProductVersion      _T(Averstr_ProductVersion  )
#define Tverstr_SpecialBuild        _T(Averstr_SpecialBuild    )

struct LangAndCodepage
{
    // Data

    WORD m_wLanguage;
    WORD m_wCodePage;

    // Methods

    LangAndCodepage()
    { m_wLanguage = m_wCodePage = 0; }

    LangAndCodepage( WORD wLang, WORD wCP )
    { m_wLanguage = wLang; m_wCodePage = wCP; }

    LangAndCodepage( DWORD dw ) { SetFromDWord(dw); }

    DWORD GetAsDWord()
    { return (DWORD) MAKELONG( m_wCodePage, m_wLanguage ); }

    void SetFromDWord( DWORD dw )
    { m_wLanguage = HIWORD(dw); m_wCodePage = LOWORD(dw); }
};

class Version
{
    protected :
        HMODULE         m_hModule;
        PBYTE           m_pFileVI;
        int             m_cFileVI;
        bool            m_bLoaded;
        LangAndCodepage m_SelectedLanguage;

    public :
        Version();
        Version( HMODULE hModule );
        ~Version() { Unload(); }

        PCSTR  GetA( PCSTR  pstrVersionString ) const;
        PCWSTR GetW( PCWSTR pstrVersionString ) const;
        PCSTR  Get ( PCSTR  pstrVersionString ) const { return GetA( pstrVersionString ); }
        PCWSTR Get ( PCWSTR pstrVersionString ) const { return GetW( pstrVersionString ); }
        int    GetBuildNr( PCTSTR pstrVersionString ) const;
            ///<summary> Gets the lanugage and codepage that have been selected as best match from <b>SelectLanguage()</b>. </summary>
        LangAndCodepage GetLanguage() const { return m_SelectedLanguage; }

        bool Load( HMODULE hModule = NULL );
        void Unload();
        bool isLoaded() const { return m_bLoaded; }

        bool SelectLanguage( LANGID lang ) { return SelectLanguage( LangAndCodepage(lang, 0xffff) ); }
        bool SelectLanguage( LangAndCodepage lang );
        bool SelectLanguageThreadLocale() { return SelectLanguage( LANGIDFROMLCID( GetThreadLocale() ) ); }

    private :
        void Ctor()
        { m_hModule = NULL; m_pFileVI = NULL; m_cFileVI = 0; m_bLoaded = false; }
};




}

#endif
